@@ -2,10 +2,10 @@
 /*
  * gsf-output.c: interface for storing data
  *
- * Copyright (C) 2002 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2002-2004 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2.1 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,7 +13,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
@@ -22,12 +22,62 @@
 #include <gsf-config.h>
 #include <gsf/gsf-output-impl.h>
 #include <gsf/gsf-impl-utils.h>
-
-#define GET_CLASS(instance) G_TYPE_INSTANCE_GET_CLASS (instance, GSF_OUTPUT_TYPE, GsfOutputClass)
+#include <string.h>
 
 static gboolean
 gsf_output_vprintf (GsfOutput *output, char const* format, va_list args);
 
+#define GET_CLASS(instance) G_TYPE_INSTANCE_GET_CLASS (instance, GSF_OUTPUT_TYPE, GsfOutputClass)
+
+static GObjectClass *parent_class;
+
+enum {
+	PROP_0,
+	PROP_NAME,
+	PROP_SIZE,
+	PROP_CLOSED,
+	PROP_POS
+};
+
+static void
+gsf_output_set_property (GObject      *object,
+			 guint         property_id,
+	 G_GNUC_UNUSED   GValue const *value,
+			 GParamSpec   *pspec)
+{
+	switch (property_id) {
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+	}
+}
+
+static void
+gsf_output_get_property (GObject     *object,
+			 guint        property_id,
+			 GValue      *value,
+			 GParamSpec  *pspec)
+{
+	/* gsf_off_t is typedef'd to gint64 */
+	switch (property_id) {
+	case PROP_NAME:
+		g_value_set_string (value, gsf_output_name (GSF_OUTPUT (object)));
+		break;
+	case PROP_SIZE:
+		g_value_set_int64 (value, gsf_output_size (GSF_OUTPUT (object)));
+		break;
+	case PROP_POS:
+		g_value_set_int64 (value, gsf_output_tell (GSF_OUTPUT (object)));
+		break;
+	case PROP_CLOSED:
+		g_value_set_boolean (value, gsf_output_is_closed (GSF_OUTPUT (object)));
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+	}
+}
+
 static void
 gsf_output_finalize (GObject *obj)
 {
@@ -51,6 +101,8 @@ gsf_output_finalize (GObject *obj)
 		output->container = NULL;
 	}
 	g_clear_error (&output->err);
+
+	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 static void
@@ -74,8 +126,33 @@ gsf_output_class_init (GObjectClass *gobject_class)
 {
 	GsfOutputClass  *output_class  = GSF_OUTPUT_CLASS (gobject_class);
 
-	gobject_class->finalize = gsf_output_finalize;
-	output_class->Vprintf   = gsf_output_vprintf;
+	gobject_class->finalize     = gsf_output_finalize;
+	gobject_class->set_property = gsf_output_set_property;
+	gobject_class->get_property = gsf_output_get_property;
+	output_class->Vprintf       = gsf_output_vprintf;
+
+	parent_class = g_type_class_peek_parent (gobject_class);
+
+	g_object_class_install_property (gobject_class,
+					 PROP_NAME,
+					 g_param_spec_pointer ("name", "Name",
+							       "The Output's Name",
+							       G_PARAM_READABLE));
+	g_object_class_install_property (gobject_class,
+					 PROP_SIZE,
+					 g_param_spec_pointer ("size", "Size",
+							       "The Output's Size",
+							       G_PARAM_READABLE));
+	g_object_class_install_property (gobject_class,
+					 PROP_POS,
+					 g_param_spec_pointer ("position", "Position",
+							       "The Output's Current Position",
+							       G_PARAM_READABLE));
+	g_object_class_install_property (gobject_class,
+					 PROP_CLOSED,
+					 g_param_spec_pointer ("is_closed", "Is Closed",
+							       "Whether the Output is Closed",
+							       G_PARAM_READABLE));
 }
 
 GSF_CLASS_ABSTRACT (GsfOutput, gsf_output,
@@ -296,7 +373,7 @@ gsf_output_set_name (GsfOutput *output, char const *name)
  * outputs.
  *
  * Returns : TRUE if the assignment was ok.
- */
+ **/
 gboolean
 gsf_output_set_container (GsfOutput *output, GsfOutfile *container)
 {
@@ -313,13 +390,15 @@ gsf_output_set_container (GsfOutput *output, GsfOutfile *container)
 /**
  * gsf_output_set_error :
  * @output :
- * @container :
+ * @code :
+ * @format :
+ * @Varargs :
  *
  * <protected> This is a utility routine that should only be used by derived
  * outputs.
  *
- * Returns : Always returns FALSE to facilitate its use.
- */
+ * Returns Always returns FALSE to facilitate its use.
+ **/
 gboolean
 gsf_output_set_error (GsfOutput  *output,
 		      gint        code,
@@ -344,10 +423,9 @@ gsf_output_set_error (GsfOutput  *output,
 }
 
 static void
-cb_output_wrap_screwup (gpointer wrapee, GObject *wrapper)
+cb_output_unwrap (GsfOutput *wrapee, G_GNUC_UNUSED GObject *wrapper)
 {
-	g_warning ("%p died while still claiming ownership of %p",
-		   wrapper, wrapee);
+	wrapee->wrapped_by = NULL;
 }
 
 /**
@@ -358,7 +436,7 @@ cb_output_wrap_screwup (gpointer wrapee, GObject *wrapper)
  * Returns TRUE if the wrapping succeeded.
  **/
 gboolean
-gsf_output_wrap (GsfOutput *wrapper, GsfOutput *wrapee)
+gsf_output_wrap (GObject *wrapper, GsfOutput *wrapee)
 {
 	g_return_val_if_fail (wrapper != NULL, FALSE);
 	g_return_val_if_fail (wrapee != NULL, FALSE);
@@ -368,8 +446,8 @@ gsf_output_wrap (GsfOutput *wrapper, GsfOutput *wrapee)
 		return FALSE;
 	}
 
-	/* worry about someone dying with out unwrapping */
-	g_object_weak_ref (G_OBJECT (wrapper), cb_output_wrap_screwup, wrapee);
+	g_object_weak_ref (G_OBJECT (wrapper),
+		(GWeakNotify) cb_output_unwrap, wrapee);
 	wrapee->wrapped_by = wrapper;
 	return TRUE;
 }
@@ -379,16 +457,17 @@ gsf_output_wrap (GsfOutput *wrapper, GsfOutput *wrapee)
  * @wrapper :
  * @wrapee :
  *
- * Returns TRUE if the wrapping succeeded.
+ * Returns TRUE if the unwrapping succeeded.
  **/
 gboolean
-gsf_output_unwrap (GsfOutput *wrapper, GsfOutput *wrapee)
+gsf_output_unwrap (GObject *wrapper, GsfOutput *wrapee)
 {
 	g_return_val_if_fail (wrapee != NULL, FALSE);
 	g_return_val_if_fail (wrapee->wrapped_by == wrapper, FALSE);
 
 	wrapee->wrapped_by = NULL;
-	g_object_weak_unref (G_OBJECT (wrapper), cb_output_wrap_screwup, wrapee);
+	g_object_weak_unref (G_OBJECT (wrapper),
+		(GWeakNotify) cb_output_unwrap, wrapee);
 	return TRUE;
 }
 
@@ -408,7 +487,7 @@ gsf_output_vprintf (GsfOutput *output, char const *fmt, va_list args)
 
 	if (NULL == output->printf_buf) {
 		output->printf_buf_size = 128;
-		output->printf_buf = g_malloc (output->printf_buf_size);
+		output->printf_buf = g_new (char, output->printf_buf_size);
 	}
 	reslen = g_vsnprintf (output->printf_buf, output->printf_buf_size, fmt, args);
 
@@ -422,8 +501,16 @@ gsf_output_vprintf (GsfOutput *output, char const *fmt, va_list args)
 	return gsf_output_write (output, reslen, output->printf_buf);
 }
 
+/**
+ * gsf_output_printf:
+ * @output: A #GsfOutput
+ * @format: The printf-style format string
+ * @Varargs: the arguments for @format
+ *
+ * Returns: TRUE if successful, FALSE if not
+ **/
 gboolean
-gsf_output_printf (GsfOutput *output, char const* format, ...)
+gsf_output_printf (GsfOutput *output, char const *format, ...)
 {
 	va_list args;
 	gboolean ret;
@@ -439,9 +526,17 @@ gsf_output_printf (GsfOutput *output, char const* format, ...)
 	return ret;
 }
 
-/* Like fputs, this assumes that the line already ends with a newline */
+/**
+ * gsf_output_puts:
+ * @output: A #GsfOutput
+ * @line: %null terminated string to write
+ *
+ * Like fputs, this assumes that the line already ends with a newline 
+ *
+ * Returns: TRUE if successful, FALSE if not
+ **/
 gboolean
-gsf_output_puts (GsfOutput *output, char const* line)
+gsf_output_puts (GsfOutput *output, char const *line)
 {
 	size_t nbytes = 0;
 
